@@ -15,7 +15,7 @@
   | Author: Sara Golemon <pollita@php.net>                               |
   +----------------------------------------------------------------------+
 
-  $Id: ssh2_sftp.c 305068 2010-11-03 22:24:19Z pajoye $
+  $Id: ssh2_sftp.c 317118 2011-09-21 19:05:21Z bjori $
 */
 
 #ifdef HAVE_CONFIG_H
@@ -81,6 +81,7 @@ inline int php_ssh2_sftp_attr2ssb(php_stream_statbuf *ssb, LIBSSH2_SFTP_ATTRIBUT
 	if (attrs->flags & LIBSSH2_SFTP_ATTR_SIZE) {
 		ssb->sb.st_size = attrs->filesize;
 	}
+	
 	if (attrs->flags & LIBSSH2_SFTP_ATTR_UIDGID) {
 		ssb->sb.st_uid = attrs->uid;
 		ssb->sb.st_gid = attrs->gid;
@@ -107,8 +108,11 @@ typedef struct _php_ssh2_sftp_handle_data {
 static size_t php_ssh2_sftp_stream_write(php_stream *stream, const char *buf, size_t count TSRMLS_DC)
 {
 	php_ssh2_sftp_handle_data *data = (php_ssh2_sftp_handle_data*)stream->abstract;
+	ssize_t bytes_written;
 
-	return libssh2_sftp_write(data->handle, buf, count);
+	bytes_written = libssh2_sftp_write(data->handle, buf, count);
+
+	return (size_t)(bytes_written<0 ? 0 : bytes_written);
 }
 /* }}} */
 
@@ -117,8 +121,13 @@ static size_t php_ssh2_sftp_stream_write(php_stream *stream, const char *buf, si
 static size_t php_ssh2_sftp_stream_read(php_stream *stream, char *buf, size_t count TSRMLS_DC)
 {
 	php_ssh2_sftp_handle_data *data = (php_ssh2_sftp_handle_data*)stream->abstract;
+	ssize_t bytes_read;
 
-	return libssh2_sftp_read(data->handle, buf, count);
+	bytes_read = libssh2_sftp_read(data->handle, buf, count);
+
+	stream->eof = (bytes_read <= 0 && bytes_read != LIBSSH2_ERROR_EAGAIN);
+
+	return (size_t)(bytes_read<0 ? 0 : bytes_read);
 }
 /* }}} */
 
@@ -262,7 +271,7 @@ static size_t php_ssh2_sftp_dirstream_read(php_stream *stream, char *buf, size_t
 	php_stream_dirent *ent = (php_stream_dirent*)buf;
 	size_t bytesread = libssh2_sftp_readdir(data->handle, ent->d_name, sizeof(ent->d_name) - 1, NULL);
 	char *basename = NULL;
-	int basename_len = 0;
+	size_t basename_len = 0;
 
 	if (bytesread <= 0) {
 		return 0;
@@ -291,14 +300,6 @@ static size_t php_ssh2_sftp_dirstream_read(php_stream *stream, char *buf, size_t
 	ent->d_name[bytesread] = 0;
 	efree(basename);
 
-	/* Trim off trailing whitespace characters */
-	bytesread--;
-	while (bytesread >= 0 &&
-			(ent->d_name[bytesread] == '\n' || ent->d_name[bytesread] == '\r' ||
-			 ent->d_name[bytesread] == '\t' || ent->d_name[bytesread] == ' ')) {
-		ent->d_name[bytesread--] = '\0';
-	}
-
 	return sizeof(php_stream_dirent);
 }
 /* }}} */
@@ -677,8 +678,11 @@ PHP_FUNCTION(ssh2_sftp_mkdir)
 		/* Just attempt to make every directory, some will fail, but we only care about the last success/failure */
 		p = filename;
 		while ((p = strchr(p + 1, '/'))) {
+			if ((p - filename) + 1 == filename_len) {
+				break;
+			}
 			libssh2_sftp_mkdir_ex(data->sftp, filename, p - filename, mode);
-		}	
+		}
 	}
 
 
